@@ -12,6 +12,7 @@
 #include <inputmode/kateviinputmode.h>
 #include <kateconfig.h>
 #include <katedocument.h>
+#include <kateglobal.h>
 #include <kateundomanager.h>
 #include <kateview.h>
 #include <vimode/emulatedcommandbar/emulatedcommandbar.h>
@@ -28,6 +29,9 @@ using namespace KTextEditor;
 
 BaseTest::BaseTest()
 {
+    // test mode with e.g. own temporary config files
+    KTextEditor::EditorPrivate::enableUnitTestMode();
+
     kate_view = nullptr;
     kate_document = nullptr;
 
@@ -245,8 +249,8 @@ void BaseTest::DoTest_(int line,
                        Expectation expectation,
                        const char *failureReason)
 {
-    BeginTest(QLatin1String(original));
-    TestPressKey(QLatin1String(command));
+    BeginTest(QString::fromUtf8(original));
+    TestPressKey(QString::fromUtf8(command));
     FinishTest_(line, file, expected, expectation, failureReason);
 }
 
