@@ -47,9 +47,20 @@ TextLine TextBlock::line(int line) const
     return m_lines.at(line - startLine());
 }
 
+void TextBlock::setLineMetaData(int line, const TextLine &textLine)
+{
+    // right input
+    Q_ASSERT(line >= startLine());
+
+    // set stuff, at will bail out on out-of-range
+    const QString originalText = m_lines.at(line - startLine()).text();
+    m_lines.at(line - startLine()) = textLine;
+    m_lines.at(line - startLine()).text() = originalText;
+}
+
 void TextBlock::appendLine(const QString &textOfLine)
 {
-    m_lines.push_back(std::make_shared<Kate::TextLineData>(textOfLine));
+    m_lines.emplace_back(textOfLine);
     m_blockSize += textOfLine.size();
 }
 
@@ -68,7 +79,7 @@ void TextBlock::text(QString &text) const
             text.append(QLatin1Char('\n'));
         }
 
-        text.append(m_lines.at(i)->text());
+        text.append(m_lines.at(i).text());
     }
 }
 
@@ -77,36 +88,36 @@ void TextBlock::wrapLine(const KTextEditor::Cursor position, int fixStartLinesSt
     // calc internal line
     int line = position.line() - startLine();
 
-    // get text
-    QString &text = m_lines.at(line)->textReadWrite();
+    // get text, copy, we might invalidate the reference
+    const QString text = m_lines.at(line).text();
 
     // check if valid column
     Q_ASSERT(position.column() >= 0);
     Q_ASSERT(position.column() <= text.size());
 
     // create new line and insert it
-    m_lines.insert(m_lines.begin() + line + 1, TextLine(new TextLineData()));
+    m_lines.insert(m_lines.begin() + line + 1, TextLine());
 
     // cases for modification:
     // 1. line is wrapped in the middle
     // 2. if empty line is wrapped, mark new line as modified
     // 3. line-to-be-wrapped is already modified
-    if (position.column() > 0 || text.size() == 0 || m_lines.at(line)->markedAsModified()) {
-        m_lines.at(line + 1)->markAsModified(true);
-    } else if (m_lines.at(line)->markedAsSavedOnDisk()) {
-        m_lines.at(line + 1)->markAsSavedOnDisk(true);
+    if (position.column() > 0 || text.size() == 0 || m_lines.at(line).markedAsModified()) {
+        m_lines.at(line + 1).markAsModified(true);
+    } else if (m_lines.at(line).markedAsSavedOnDisk()) {
+        m_lines.at(line + 1).markAsSavedOnDisk(true);
     }
 
     // perhaps remove some text from previous line and append it
     if (position.column() < text.size()) {
         // text from old line moved first to new one
-        m_lines.at(line + 1)->textReadWrite() = text.right(text.size() - position.column());
+        m_lines.at(line + 1).text() = text.right(text.size() - position.column());
 
         // now remove wrapped text from old line
-        text.chop(text.size() - position.column());
+        m_lines.at(line).text().chop(text.size() - position.column());
 
         // mark line as modified
-        m_lines.at(line)->markAsModified(true);
+        m_lines.at(line).markAsModified(true);
     }
 
     // fix all start lines
@@ -191,19 +202,18 @@ void TextBlock::unwrapLine(int line, TextBlock *previousBlock, int fixStartLines
         Q_ASSERT(previousBlock->lines() > 0);
 
         // move last line of previous block to this one, might result in empty block
-        TextLine oldFirst = m_lines.at(0);
+        const TextLine oldFirst = m_lines.at(0);
         int lastLineOfPreviousBlock = previousBlock->lines() - 1;
-        TextLine newFirst = previousBlock->m_lines.back();
-        m_lines[0] = newFirst;
+        m_lines[0] = previousBlock->m_lines.back();
         previousBlock->m_lines.erase(previousBlock->m_lines.begin() + (previousBlock->lines() - 1));
 
-        const int oldSizeOfPreviousLine = newFirst->text().size();
-        if (oldFirst->length() > 0) {
+        const int oldSizeOfPreviousLine = m_lines[0].text().size();
+        if (oldFirst.length() > 0) {
             // append text
-            newFirst->textReadWrite().append(oldFirst->text());
+            m_lines[0].text().append(oldFirst.text());
 
             // mark line as modified, since text was appended
-            newFirst->markAsModified(true);
+            m_lines[0].markAsModified(true);
         }
 
         // patch startLine of this block
@@ -282,17 +292,17 @@ void TextBlock::unwrapLine(int line, TextBlock *previousBlock, int fixStartLines
     }
 
     // easy: just move text to previous line and remove current one
-    const int oldSizeOfPreviousLine = m_lines.at(line - 1)->length();
-    const int sizeOfCurrentLine = m_lines.at(line)->length();
+    const int oldSizeOfPreviousLine = m_lines.at(line - 1).length();
+    const int sizeOfCurrentLine = m_lines.at(line).length();
     if (sizeOfCurrentLine > 0) {
-        m_lines.at(line - 1)->textReadWrite().append(m_lines.at(line)->text());
+        m_lines.at(line - 1).text().append(m_lines.at(line).text());
     }
 
-    const bool lineChanged = (oldSizeOfPreviousLine > 0 && m_lines.at(line - 1)->markedAsModified())
-        || (sizeOfCurrentLine > 0 && (oldSizeOfPreviousLine > 0 || m_lines.at(line)->markedAsModified()));
-    m_lines.at(line - 1)->markAsModified(lineChanged);
-    if (oldSizeOfPreviousLine == 0 && m_lines.at(line)->markedAsSavedOnDisk()) {
-        m_lines.at(line - 1)->markAsSavedOnDisk(true);
+    const bool lineChanged = (oldSizeOfPreviousLine > 0 && m_lines.at(line - 1).markedAsModified())
+        || (sizeOfCurrentLine > 0 && (oldSizeOfPreviousLine > 0 || m_lines.at(line).markedAsModified()));
+    m_lines.at(line - 1).markAsModified(lineChanged);
+    if (oldSizeOfPreviousLine == 0 && m_lines.at(line).markedAsSavedOnDisk()) {
+        m_lines.at(line - 1).markAsSavedOnDisk(true);
     }
 
     m_lines.erase(m_lines.begin() + line);
@@ -356,9 +366,9 @@ void TextBlock::insertText(const KTextEditor::Cursor position, const QString &te
     int line = position.line() - startLine();
 
     // get text
-    QString &textOfLine = m_lines.at(line)->textReadWrite();
+    QString &textOfLine = m_lines.at(line).text();
     int oldLength = textOfLine.size();
-    m_lines.at(line)->markAsModified(true);
+    m_lines.at(line).markAsModified(true);
 
     // check if valid column
     Q_ASSERT(position.column() >= 0);
@@ -427,7 +437,7 @@ void TextBlock::removeText(KTextEditor::Range range, QString &removedText)
     int line = range.start().line() - startLine();
 
     // get text
-    QString &textOfLine = m_lines.at(line)->textReadWrite();
+    QString &textOfLine = m_lines.at(line).text();
     int oldLength = textOfLine.size();
 
     // check if valid column
@@ -441,7 +451,7 @@ void TextBlock::removeText(KTextEditor::Range range, QString &removedText)
 
     // remove text
     textOfLine.remove(range.start().column(), range.end().column() - range.start().column());
-    m_lines.at(line)->markAsModified(true);
+    m_lines.at(line).markAsModified(true);
 
     // notify the text history
     m_buffer->history().removeText(range, oldLength);
@@ -499,8 +509,8 @@ void TextBlock::debugPrint(int blockIndex) const
         printf("%4d - %4llu : %4llu : '%s'\n",
                blockIndex,
                (unsigned long long)startLine() + i,
-               (unsigned long long)m_lines.at(i)->text().size(),
-               qPrintable(m_lines.at(i)->text()));
+               (unsigned long long)m_lines.at(i).text().size(),
+               qPrintable(m_lines.at(i).text()));
     }
 }
 
@@ -516,8 +526,8 @@ TextBlock *TextBlock::splitBlock(int fromLine)
     newBlock->m_lines.reserve(linesOfNewBlock);
     for (size_t i = fromLine; i < m_lines.size(); ++i) {
         auto line = std::move(m_lines[i]);
-        m_blockSize -= line->length();
-        newBlock->m_blockSize += line->length();
+        m_blockSize -= line.length();
+        newBlock->m_blockSize += line.length();
         newBlock->m_lines.push_back(std::move(line));
     }
 
@@ -684,8 +694,8 @@ void TextBlock::markModifiedLinesAsSaved()
 {
     // mark all modified lines as saved
     for (auto &textLine : m_lines) {
-        if (textLine->markedAsModified()) {
-            textLine->markAsSavedOnDisk(true);
+        if (textLine.markedAsModified()) {
+            textLine.markAsSavedOnDisk(true);
         }
     }
 }
