@@ -37,8 +37,9 @@ public:
      * Construct file loader for given file.
      * @param filename file to open
      * @param proberType prober type
+     * @param lineLengthLimit limit for lines to load, else we break them up in smaller ones
      */
-    TextLoader(const QString &filename, KEncodingProber::ProberType proberType)
+    TextLoader(const QString &filename, KEncodingProber::ProberType proberType, int lineLengthLimit)
         : m_eof(false) // default to not eof
         , m_lastWasEndOfLine(true) // at start of file, we had a virtual newline
         , m_lastWasR(false) // we have not found a \r as last char
@@ -51,6 +52,7 @@ public:
         , m_firstRead(true)
         , m_proberType(proberType)
         , m_fileSize(0)
+        , m_lineLengthLimit(lineLengthLimit)
     {
         // try to get mimetype for on the fly decompression, don't rely on filename!
         QFile testMime(filename);
@@ -163,9 +165,11 @@ public:
      * read a line, return length + offset in Unicode data
      * @param offset offset into internal Unicode data for read line
      * @param length length of read line
+     * @param tooLongLinesWrapped was a too long line seen?
+     * @param longestLineLoaded length of the longest line that hit the limit
      * @return true if no encoding errors occurred
      */
-    bool readLine(int &offset, int &length)
+    bool readLine(int &offset, int &length, bool &tooLongLinesWrapped, int &longestLineLoaded)
     {
         length = 0;
         offset = 0;
@@ -180,16 +184,54 @@ public:
          * bug 272579
          */
         bool failedToConvertOnce = false;
+
         /**
          * keep track if we have found BOM so that failedToConvertOnce is not erroneously set to true
          * BUG: 440359
          */
         bool bomPreviouslyFound = m_bomFound;
 
+        // honor the line length limit early
+        const auto lineLimitHandler = [this, &offset, &length, &tooLongLinesWrapped, &longestLineLoaded](int lineStart, int textLength) {
+            if ((m_lineLengthLimit <= 0) || (textLength <= m_lineLengthLimit)) {
+                return false;
+            }
+
+            // remember stick error
+            tooLongLinesWrapped = true;
+            longestLineLoaded = std::max(longestLineLoaded, textLength);
+
+            // search for place to wrap
+            int spacePosition = m_lineLengthLimit - 1;
+            for (int testPosition = m_lineLengthLimit - 1; (testPosition >= 0) && (testPosition >= (m_lineLengthLimit - (m_lineLengthLimit / 10)));
+                 --testPosition) {
+                // wrap place found?
+                if (m_text[lineStart + testPosition].isSpace() || m_text[lineStart + testPosition].isPunct()) {
+                    spacePosition = testPosition;
+                    break;
+                }
+            }
+
+            m_lastWasEndOfLine = false;
+            m_lastWasR = false;
+
+            // line data
+            offset = lineStart;
+            length = spacePosition + 1;
+
+            m_lastLineStart = m_position = (lineStart + length);
+            return true;
+        };
+
         /**
          * reading loop
          */
         while (m_position <= m_text.length()) {
+            // handle too long lines early even if we not yet have seen the end
+            if (m_alreadyScanned > m_lastLineStart && lineLimitHandler(m_lastLineStart, m_alreadyScanned - m_lastLineStart)) {
+                return !encodingError;
+            }
+
             if (m_position == m_text.length()) {
                 // try to load more text if something is around
                 if (!m_eof) {
@@ -264,6 +306,7 @@ public:
 
                     // recalc current pos and last pos
                     m_position -= m_lastLineStart;
+                    m_alreadyScanned = m_position - 1;
                     m_lastLineStart = 0;
                 }
 
@@ -277,6 +320,7 @@ public:
 
                     m_lastLineStart = m_position;
 
+                    lineLimitHandler(offset, length);
                     return !encodingError && !failedToConvertOnce;
                 }
 
@@ -291,7 +335,9 @@ public:
                     continue;
                 }
             }
+
             for (; m_position < m_text.length(); m_position++) {
+                m_alreadyScanned = m_position;
                 QChar current_char = m_text.at(m_position);
                 if (current_char == lf) {
                     m_lastWasEndOfLine = true;
@@ -313,6 +359,7 @@ public:
                             m_eol = TextBuffer::eolUnix;
                         }
 
+                        lineLimitHandler(offset, length);
                         return !encodingError;
                     }
                 } else if (current_char == cr) {
@@ -331,6 +378,7 @@ public:
                         m_eol = TextBuffer::eolMac;
                     }
 
+                    lineLimitHandler(offset, length);
                     return !encodingError;
                 } else if (current_char == QChar::LineSeparator) {
                     m_lastWasEndOfLine = true;
@@ -342,6 +390,7 @@ public:
                     m_lastLineStart = m_position + 1;
                     m_position++;
 
+                    lineLimitHandler(offset, length);
                     return !encodingError;
                 } else {
                     m_lastWasEndOfLine = false;
@@ -365,6 +414,7 @@ private:
     bool m_lastWasR;
     int m_position;
     int m_lastLineStart;
+    int m_alreadyScanned = -1;
     TextBuffer::EndOfLineMode m_eol;
     QString m_mimeType;
     QIODevice *m_file;
@@ -376,6 +426,7 @@ private:
     bool m_firstRead;
     KEncodingProber::ProberType m_proberType;
     quint64 m_fileSize;
+    const int m_lineLengthLimit;
 };
 
 }
