@@ -7,32 +7,21 @@
 #ifndef KATEARGUMENTHINTMODEL_H
 #define KATEARGUMENTHINTMODEL_H
 
-#include "expandingtree/expandingwidgetmodel.h"
 #include "katecompletionmodel.h"
 #include <QAbstractListModel>
 
 class KateCompletionWidget;
 
-class KateArgumentHintModel : public ExpandingWidgetModel
+class KateArgumentHintModel : public QAbstractListModel
 {
     Q_OBJECT
 public:
-    explicit KateArgumentHintModel(KateCompletionWidget *parent);
+    explicit KateArgumentHintModel(KateCompletionModel *parent);
 
     QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const override;
 
     int rowCount(const QModelIndex &parent = {}) const override;
 
-    int columnCount(const QModelIndex &parent = {}) const override;
-
-    QModelIndex index(int row, int column, const QModelIndex &parent = {}) const override;
-
-    QModelIndex parent(const QModelIndex &parent) const override;
-
-    QTreeView *treeView() const override;
-
-    bool indexIsItem(const QModelIndex &index) const override;
-
     void emitDataChanged(const QModelIndex &start, const QModelIndex &end);
 
     // Returns the index in the source-model for an index within this model
@@ -41,20 +30,17 @@ public:
     void buildRows();
     void clear();
 
-protected:
-    int contextMatchQuality(const QModelIndex &row) const override;
 public Q_SLOTS:
     void parentModelReset();
+
 Q_SIGNALS:
     void contentStateChanged(bool hasContent);
 
 private:
     KateCompletionModel::Group *group() const;
-    KateCompletionModel *model() const;
 
     std::vector<int> m_rows; // Maps rows to either a positive row-number in the source group, or to a negative number which indicates a label
-
-    KateCompletionWidget *m_parent;
+    KateCompletionModel *m_parent;
 };
 
 #endif
