@@ -15,6 +15,7 @@
 #include "kateviewhelpers.h"
 
 #include <QJsonObject>
+#include <QMenu>
 
 #include <KActionCollection>
 #include <KLocalizedString>
@@ -134,5 +135,3 @@ void KateScriptActionMenu::repopulate()
 }
 
 // END KateScriptActionMenu
-
-#include "moc_katescriptaction.cpp"
